#!/usr/bin/env python2
from __future__ import print_function


"""netdev-info.py: parse output from `ip addr`"""
__version__   = "1.0"
__author__    = "ed <netdevinfo.py@ocv.me>"
__credits__   = ["stackoverflow.com"]
__license__   = "MIT"
__copyright__ = 2015  # wow has it been that long


import re
import os
import sys
import subprocess as sp


def runcmd(*argv):
	p = sp.Popen(argv, stdout=sp.PIPE, stderr=sp.PIPE)
	stdout, stderr = p.communicate()
	try:
		stdout = stdout.decode('utf-8')
		stderr = stderr.decode('utf-8')
	except:
		pass
	
	return [p.returncode, stdout, stderr]


def get_netinfo():
	# 2: eth0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP group default qlen 1000  //wan-ip
	# 3: qemu_keet2: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue state UP group default qlen 1000  //int-ip
	# 8: tap0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast master qemu_keet2 state UNKNOWN group default qlen 1000  //no-ip
	r_dev = re.compile(r'^[0-9]+: ([^ :]+): <([A-Z,_-]+)>( .*)')
	r_mac = re.compile(r'^ +link/ether ([0-9a-f:]{17}) ')
	r_pf4 = re.compile(r'^ +inet ([0-9\.]{7,15}/[0-9]{1,2}) ')
	
	txt = runcmd('ip', 'addr')[1]
	dev = None
	devs = {}
	for ln in txt.split('\n'):
		
		m = r_dev.match(ln)
		if m:
			name = m.group(1)
			flags = m.group(2).split(',')
			kv = m.group(3)[1:].split(' ')
			kv = dict(zip(kv[::2], kv[1::2]))
			
			parent = None
			if 'master' in kv:
				parent = kv['master']
			
			dev = {
				'pf4': [],
				'ip4': [],
				'name': m.group(1),
				'link': kv['state']=='UP',
				'parent': parent,
				'children': []
			}
			
			if name != 'lo':
				devs[name] = dev
		
		m = r_mac.match(ln)
		if m:
			dev['mac'] = m.group(1)
			continue
		
		m = r_pf4.match(ln)
		if m:
			dev['pf4'].append(m.group(1))
			dev['ip4'].append(m.group(1).split('/')[0])
			continue
	
	# copy children into parents
	for _, dev in devs.items():
		parent = dev['parent']
		if parent:
			parent = devs[parent]
			parent['children'].append(dev)
	
	eth = {}  # plain interface (no children/parent)
	bri = {}  # bridges (have children)
	tap = {}  # taps (have parents)
	
	for _, dev in devs.items():
		if dev['children']:
			bri[dev['name']] = dev
		elif dev['parent']:
			tap[dev['name']] = dev
		else:
			eth[dev['name']] = dev
	
	return [eth,bri,tap]


def print_netinfo_ifs(ifs):
	msg = ''
	for _, dev in sorted(ifs.items()):
		leading = '\033[0m, ' if msg else '  '
		markup = '0;36m' if dev['link'] else '1;31m--'
		msg += '{0}\033[{1}{2}'.format(
			leading, markup, dev['name'])

	msg += '\033[0m'
	print(msg)


def print_netinfo(ni):
	eth, bri, tap = ni
	
	if eth:
		print('regular interfaces:')
		print_netinfo_ifs(eth)
		print()
	
	if bri:
		print('bridges:')
		print_netinfo_ifs(bri)
		print()
	
	if tap:
		print('taps:')
		print_netinfo_ifs(tap)
		print()


def is_remote():
	ssh_path = os.getenv('SSH_CONNECTION')
	if ssh_path:
		ssh_nodes = ssh_path.split(' ')[::2]
		return '127.0.0.1' not in ssh_nodes \
			and ssh_nodes[0] != ssh_nodes[1]
	
	return False


print('\nfor the list of detected interfaces below,\ncable connection status is indicated by\n  \033[0;36mhaveCarrier\033[0m and \033[1;31m--nopeCarrier\033[0m\n')
print_netinfo(get_netinfo())


print('and judging by the endpoints in SSH_CONNECTION (if any),')
if is_remote():
	print('\033[33m  you are connecting from a remote computer')
else:
	print('\033[32m  this appears to be a local shell')
